# coding=utf-8
from __future__ import absolute_import, print_function

from suanpan import asyncio
from suanpan.g import g
from suanpan.run import cli, helper, env, run
from suanpan.sfile import settings

__version__ = "0.19.8"
__all__ = ['asyncio', 'g', 'cli', 'helper', 'env', 'run', 'settings']

if __name__ == "__main__":
    print(f"Suanpan SDK (ver: {__version__})")
